/****************************************************************************
**
** Copyright (C) 2006-2009 fullmetalcoder <fullmetalcoder@hotmail.fr>
**
** This file is part of the Edyuk project <http://edyuk.org>
** 
** This file may be used under the terms of the GNU General Public License
** version 3 as published by the Free Software Foundation and appearing in the
** file GPL.txt included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef _CODE_DEFINITION_H_
#define _CODE_DEFINITION_H_

#include "qce-config.h"

/*!
	\file codedefinition.h
	\brief Definition of the CodeDefinition class.
	
	\see CodeDefinition
*/

#include "qlanguagefactory.h"
#include "qlanguagedefinition.h"

struct HashedAutomaton;

class CodeDefinition : public QLanguageDefinition
{
	public:
		enum ContextFlags
		{
			None,
			Merge		= 1,
			Hide		= 2
		};
		
		CodeDefinition(HashedAutomaton *a);
		virtual ~CodeDefinition();
		
		virtual QString language() const;
		virtual QStringList extensions() const;
		
		virtual int tokenize(QDocument *d, int firstLine, int count);
		
		virtual QString singleLineComment() const;
		
		virtual QString defaultLineMark() const;
		
		virtual void match(QDocumentCursor& c);
		virtual void clearMatches(QDocument *d);
		
		virtual QString indent(const QDocumentCursor& c);
		virtual bool unindent (const QDocumentCursor& c, const QString& ktxt);
		
		virtual void expand(QDocument *d, int line);
		virtual void collapse(QDocument *d, int line);
		virtual int blockFlags(QDocument *d, int line, int depth = 0) const;
		
		static void load(const QString& file, QLanguageFactory::LangData *d, QFormatScheme *s);
		
	private:
		void registerContext(HashedAutomaton *a);
		
		QString m_lang;
		QStringList m_exts;
		HashedAutomaton *m_lexer;
		QVector<HashedAutomaton*> m_contexts;
		QHash<HashedAutomaton*, int> m_contextIds;
};

#endif // !_CODE_DEFINITION_H_
